#include "toolbox.h"
#include <formatio.h>
#include "rsspecan.h"
#include <cvirte.h>     
#include <userint.h>
#include <visa.h>
#include "rsspecan_IQ_measurement_example.h"


#define ARRAY_SIZE    5000
#define STRING_LENGTH 512
#define WINDOW        1

/* Useful macros */
#define CHECKERR(fCal) \
    if (status = checkError((fCal)), status < VI_SUCCESS) \
        return status; else


/* Global variables */
static   ViInt32    panelHandle  = 0;
static   ViSession  instrSession = VI_NULL;
ViChar              filePath [STRING_LENGTH] = {NULL};
    

/* Utility functions declarations */
ViStatus checkError(ViStatus status);
void cleanup (ViBoolean deviceClear);
void EnableButtons(ViBoolean enable);
int readData(void);

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    if ((panelHandle = LoadPanel (0, "rsspecan_IQ_measurement_example.uir", PANEL)) < 0)
        return -1;
    
    DisplayPanel (panelHandle);
    RunUserInterface ();
    DiscardPanel (panelHandle);
    
    return 0;
}

int CVICALLBACK panelCallback (int panel, int event, void *callbackData,
        int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_CLOSE:
            /* cleanup specan driver session */
            cleanup(VI_FALSE);
            
            QuitUserInterface (0);
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Start Button Callback
/* Purpose:  Configures the instrument and initiates the measurement
/*===========================================================================*/
int CVICALLBACK saveButton (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViChar      instr_desc [STRING_LENGTH] = "";
    ViInt32     id               = 0;
    ViInt32     reset            = VI_FALSE;
    ViInt32     display          = VI_FALSE;
    ViStatus    status           = VI_SUCCESS;
	ViReal64    centerFreq, refLevel;
    
    switch (event)
    {
        case EVENT_COMMIT:     /* start button pressed */
            {
            
                /* block mouse events */
                SetWaitCursor (1);
                
                /* read values from panel*/
                GetCtrlVal (panelHandle, PANEL_INSTR_DESC, instr_desc);
                GetCtrlVal (panelHandle, PANEL_IDQUERY, &id);
                GetCtrlVal (panelHandle, PANEL_RESET, &reset);
                GetCtrlVal (panelHandle, PANEL_DISPLAYON, &display);
				GetCtrlVal (panelHandle, PANEL_CENTERFREQ, &centerFreq);
                GetCtrlVal (panelHandle, PANEL_REFLEVEL, &refLevel);

                /* disable buttons for configuration */
                EnableButtons(VI_FALSE);
                
                /* initialize driver */
                CHECKERR (rsspecan_init (instr_desc, id, reset, &instrSession));
				
				/* switch device display on/off */
                CHECKERR( rsspecan_ConfigureDisplayUpdate(instrSession, display));
                
                /* Set instrument to single sweep mode - continous mode is done
                 * programmatically because this is the only way how to
                 * synchronize measurement using *OPC
                 */
                //CHECKERR (rsspecan_ConfigureSignalStatisticSweep(instrSession, VI_FALSE));
                
				/* configure the center frequency */
				CHECKERR (rsspecan_ConfigureFrequencyCenter (instrSession, WINDOW, centerFreq));

				/* configure the reference level */
				CHECKERR (rsspecan_ConfigureReferenceLevel (instrSession, WINDOW, refLevel));
				
				
				CHECKERR (rsspecan_ConfigureTraceIQDataAcquisition (instrSession, WINDOW,
																	VI_TRUE));
				
				
				CHECKERR (rsspecan_TraceIQSet (instrSession, WINDOW, 32.0e6,
											   RSSPECAN_VAL_TRG_IMM, RSSPECAN_VAL_NEG, 10.0e6,
											   0, 4096));
	
				
				/* read data twice and save to file */
				readData();
				
				
				/* close session */
        		cleanup(VI_FALSE);
            }
            break;
    }
    return 0;
}



/*===========================================================================*/
/* Function: Stop Button Callback
/* Purpose:  Stops the continuous measurement, in other cases has no sense
/*===========================================================================*/
int CVICALLBACK stopButton (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
         
    {
        case EVENT_COMMIT:
            
            
            /* cleanup specan driver session */
            cleanup(VI_TRUE);
                
            /* enable buttons for configuration */
            EnableButtons(VI_TRUE);
            break;
    }
    return 0;
}



/*===========================================================================*/
/* Function: Open Button Callback
/* Purpose:  Opens file for saving measurement results
/*===========================================================================*/
int  CVICALLBACK openButton(int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
         
    {
        case EVENT_COMMIT:
            
            /* open or create new file */
			FileSelectPopup ("", "*.dat", "*.dat", "Name of File to Save",
							 VAL_OK_BUTTON, 0, 1, 1, 1, filePath);
			
			/* write file path to front panel text box */
			SetCtrlVal (panelHandle, PANEL_FILEPATH, filePath);
    }
    return 0;
}

/*===========================================================================*/
/* Function: Read Data
/* Purpose:  Reads data from the instrument and saves them into specified file
/*===========================================================================*/
int readData (void)
{
    ViStatus temp_status  = VI_SUCCESS;
	ViStatus status       = VI_SUCCESS;
	ViInt32  timeout      = 15000;               /* default value */ 
	ViInt32  actual_count = 0;
	ViInt32  numberOfPoints  = 0;
	ViReal64 data [2][ARRAY_SIZE];
	int fileHandle;
	char buffer[35];
    
	
	/* Is save file specified? */
	GetCtrlVal (panelHandle, PANEL_FILEPATH, filePath);
	if (filePath[0] == NULL | !FileExists(filePath,0))
	{
		/* open or create new file or exit if canceled */
		if (FileSelectPopup ("", "*.dat", "*.dat", "Name of File to Save",
							 VAL_OK_BUTTON, 0, 1, 1, 1, filePath) == 0)
			return -1;
			
		/* write file path to front panel text box */
		SetCtrlVal (panelHandle, PANEL_FILEPATH, filePath);
	}
	
	/* This function will start a measurement and read I/Q trace data results. */
	CHECKERR (rsspecan_ReadTraceIQData (instrSession, WINDOW, timeout, ARRAY_SIZE,
										&numberOfPoints, data[0], data[1]));
	

	/* Write header to file */
	fileHandle = OpenFile (filePath, VAL_WRITE_ONLY, VAL_TRUNCATE,
						   VAL_ASCII);
	sprintf (buffer, "Real Parts (I)\tImaginary Parts(Q)\n");
	WriteFile (fileHandle, buffer, sizeof(buffer)-1);
	
	
	/* Save measured data to file */ 
	ArrayToFile (filePath, data, VAL_DOUBLE, numberOfPoints, 2,
				 VAL_GROUPS_TOGETHER, VAL_GROUPS_AS_COLUMNS,
				 VAL_SEP_BY_TAB, 10, VAL_ASCII, VAL_APPEND);
	
	
	
	/* This function will start a measurement and read I/Q trace data results. */
	CHECKERR (rsspecan_ReadTraceIQData (instrSession, WINDOW, timeout, ARRAY_SIZE,
										&numberOfPoints, data[0], data[1]));
	
	
	/* Write header to file */
	fileHandle = OpenFile (filePath, VAL_WRITE_ONLY, VAL_APPEND,
						   VAL_ASCII);
	WriteFile (fileHandle, buffer, sizeof(buffer)-1);
	
	/* Save measured data to file */ 
	ArrayToFile (filePath, data, VAL_DOUBLE, numberOfPoints, 2,
				 VAL_GROUPS_TOGETHER, VAL_GROUPS_AS_COLUMNS,
				 VAL_SEP_BY_TAB, 10, VAL_ASCII, VAL_APPEND);
	

    return 0;
}


/*===========================================================================*/
/* Function: checkError
/* Purpose:  If passed status indicates an failure, the error message will
/*           be displayed.
/*===========================================================================*/
ViStatus checkError (ViStatus status)
{
    ViChar  error_message [STRING_LENGTH];
    ViChar  error_buffer  [STRING_LENGTH*2];
    ViChar* p2buf;
    ViInt32 error;
    
    if (status < VI_SUCCESS)
    {
        /* Converts a status code returned by an instrument driver function into a user-readable string */
        rsspecan_error_message (instrSession, status, error_message);
        p2buf = error_buffer + sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", status, error_message);
        
        
        /* This function reads an error code and a message from the instrument's error queue */
        rsspecan_error_query (instrSession, &error, error_message);
        
        SetWaitCursor (0);
        if (error != VI_SUCCESS)
        {
            sprintf (p2buf, "Secondary Error: 0x%08X, %s\n", error, error_message);
        }
        MessagePopup ("Error", error_buffer);
        
        /* close session */
        cleanup(VI_TRUE);
        
        /* enable buttons for configuration */
        EnableButtons(VI_TRUE);
    }
    return status;
}


/*===========================================================================*/
/* Function: EnableButtons
/* Purpose:  This function dis-/enables GUI while measuring/configuration.
/*===========================================================================*/
void EnableButtons(ViBoolean enable){

    SetCtrlAttribute (panelHandle, PANEL_SAVE,      ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_RESET,     ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_IDQUERY,   ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_INSTR_DESC,ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_DISPLAYON, ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_CENTERFREQ,ATTR_DIMMED,!enable);
	SetCtrlAttribute (panelHandle, PANEL_REFLEVEL,  ATTR_DIMMED,!enable);
	SetCtrlAttribute (panelHandle, PANEL_FILEPATH,  ATTR_DIMMED,!enable);
	SetCtrlAttribute (panelHandle, PANEL_OPEN,      ATTR_DIMMED,!enable);
    
    
}


/*===========================================================================*/
/* Function: Cleanup
/* Purpose:  Closes the VISA session to the instrument and in case of error
/*           queries the error and shows the dialog box with the passed error
/*           message. In case Device Clear is requested, it is necessary to 
/*           set the argument deviceClear to VI_TRUE.
/*===========================================================================*/
void cleanup (ViBoolean deviceClear)
{

    /* Device Clear is a low-level command and should be used in case the
     * instrument is waiting for Operation Complete to cancel the wait.
     * It is useful for instance in case of incorrect external trigger
     * when the instrument does not respond to any other command because of
     * waiting for trigger.
     */
    if (deviceClear)
    {
        viClear (instrSession);
        viPrintf (instrSession, "*CLS\n");
    }
    
    /* close instrument session */
    rsspecan_close (instrSession);
    instrSession = VI_NULL;
    
    /* enable GUI */
    EnableButtons (1);
    /* enable mouse events*/
    SetWaitCursor (0);
    
}


