#include <utility.h>
#include "rsspecan.h"
#include <cvirte.h>     
#include <userint.h>
#include <visa.h>
#include "rsspecan_scan_measurement.h"

#define QUEUE_SIZE                  10000
#define TRACE_POINTS_LIMIT  QUEUE_SIZE
#define ARRAY_SIZE          TRACE_POINTS_LIMIT         /* for rsspecan_ReadYTrace() varable: amplitude */
#define STRING_LENGTH       512
#define WINDOW_1            1
#define WINDOW_2            2
#define TRACE_1             1 
#define TRACE_2             2
#define SCAN_RANGE_1        1

/* Useful macros */
#define CHECKERR(fCal) \
    if (status = checkError((fCal)), status < VI_SUCCESS) \
        return status; else
#define EMI_RECEIVER "ESL"



/* Global variables */
static   ViInt32    panelHandle  = 0;
static   ViInt32    timeout      = 0;
volatile ViBoolean  enableTrace  = VI_FALSE; 
static   ViSession  instrSession = VI_NULL;
ViReal64            amplitude [ARRAY_SIZE];

ViReal64    freqstart            = 0.0;
ViReal64    freqstop             = 0.0;
ViReal64    freqstep             = 0.0;


/* Utility functions declarations */
ViStatus checkError(ViStatus status);
int ReadScan(void);
void cleanup (ViBoolean deviceClear);
void EnableButtons(ViBoolean enable);

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    if ((panelHandle = LoadPanel (0, "rsspecan_scan_measurement.uir", PANEL)) < 0)
        return -1;
    
    DisplayPanel (panelHandle);
    RunUserInterface ();

    DiscardPanel (panelHandle);
    
    return 0;
}

int CVICALLBACK panelCallback (int panel, int event, void *callbackData,
        int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_CLOSE:
            /* cleanup specan driver session */
            cleanup(VI_FALSE);
            
            QuitUserInterface (0);
            break;
    }
    return 0;
}


/*===========================================================================*/
/* Function: Start Button Callback
/* Purpose:  Configures the instrument according to selected measurement
/*           parameters and enables the measurement loop
/*===========================================================================*/
int CVICALLBACK startButton (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViChar      instr_desc [STRING_LENGTH] = "";
    ViChar      instr_modl [STRING_LENGTH] = "";
    ViInt32     id               = 0,
                i                = 0;
    ViInt32     reset            = VI_FALSE;
    ViInt32     display          = VI_FALSE;
	ViInt32		detector		 = 0;
    ViStatus    status           = VI_SUCCESS;
    
    switch (event)
    {
        case EVENT_COMMIT:     /* start button pressed */
            {
                /* block mouse events */
                SetWaitCursor (1);
                
                /* read values from panel*/
                GetCtrlVal (panelHandle, PANEL_INSTR_DESC,  instr_desc);
                GetCtrlVal (panelHandle, PANEL_IDQUERY,     &id);
                GetCtrlVal (panelHandle, PANEL_RESET,       &reset);
                GetCtrlVal (panelHandle, PANEL_START_FRQ,   &freqstart);
                GetCtrlVal (panelHandle, PANEL_STOP_FRQ,    &freqstop);
                GetCtrlVal (panelHandle, PANEL_STEP_FRQ,    &freqstep); 
                GetCtrlVal (panelHandle, PANEL_CHECKBOX,    &display);
                GetCtrlVal (panelHandle, PANEL_DETECTOR, 	&detector);

                /* disable buttons for configuration */
                EnableButtons(VI_FALSE);
                
                /* initialize driver */
                CHECKERR (rsspecan_init (instr_desc, id, reset, &instrSession));
                
				/* check for EMI test receiver */
				CHECKERR( rsspecan_GetAttributeViString (instrSession, "", RS_ATTR_INSTRUMENT_MODEL, STRING_LENGTH, instr_modl));
				if(NULL == strstr(EMI_RECEIVER, instr_modl)){
					/* enable mouse events*/
                	SetWaitCursor (0);
					/*  */
					MessagePopup ("Error", "Instrument does not support EMI scan measurements");
					/* free resource */
					cleanup(VI_TRUE);
					return -1;
				}
					
				
				/* switch to receiver mode */
                CHECKERR (rsspecan_ReceiverMode (instrSession));
                
                /* switch device display on/off */
        	    CHECKERR( rsspecan_ConfigureDisplayUpdate (instrSession, display)); 
                
                /* set one scan at 0.1 milliseconds */
	            CHECKERR (rsspecan_ConfigureScanTableMeasTime (instrSession, 1, 0.0001));
                
	           	 /* set scan step mode */
                CHECKERR (rsspecan_ConfigureReceiverStepMode (instrSession, RSSPECAN_VAL_SWEEP_SPAC_LIN));
                    
                /* switch to single sweep */
                CHECKERR (rsspecan_ConfigureAcquisition (instrSession, WINDOW_2, VI_FALSE, 1));
                
                /* for security reasons */
                CHECKERR (rsspecan_ConfigureReceiverTracePoints (instrSession, ARRAY_SIZE)); 
                
				/* Set start and stop frequency and the stepsize */
				CHECKERR (rsspecan_ConfigureScanTableStartStop(instrSession, 1, freqstart, freqstop, freqstep));
				
				/* Send trace datas before trace is finished, too */
				CHECKERR (rsspecan_ConfigureReceiverTraceFeedControl (instrSession, RSSPECAN_VAL_TRAC_FEED_ALWAYS));

				rsspecan_ConfigureFinalReceiverDetector (instrSession, 1, 1, RSSPECAN_VAL_DETECTOR_TYPE_MIN_PEAK);
				
				/* Select detector */
				if(detector == 0) // Average
				{
					CHECKERR (rsspecan_ConfigureReceiverDetector (instrSession, 1, 1, VI_TRUE, VI_FALSE, VI_FALSE, VI_FALSE, VI_FALSE, VI_FALSE, VI_FALSE));
				}
				else if(detector == 1)  // Max Peak
				{
					CHECKERR (rsspecan_ConfigureReceiverDetector (instrSession, 1, 1, VI_FALSE, VI_TRUE, VI_FALSE, VI_FALSE, VI_FALSE, VI_FALSE, VI_FALSE));
				}
				else if(detector == 2)  // Min Peak
				{
					CHECKERR (rsspecan_ConfigureReceiverDetector (instrSession, 1, 1, VI_FALSE, VI_FALSE, VI_TRUE, VI_FALSE, VI_FALSE, VI_FALSE, VI_FALSE));
				}
				else if(detector == 3)  // RMS
				{
					CHECKERR (rsspecan_ConfigureReceiverDetector (instrSession, 1, 1, VI_FALSE, VI_FALSE, VI_FALSE, VI_TRUE, VI_FALSE, VI_FALSE, VI_FALSE));
				}
				else if(detector == 4)  // QPeak
				{
					CHECKERR (rsspecan_ConfigureReceiverDetector (instrSession, 1, 1, VI_FALSE, VI_FALSE, VI_FALSE, VI_FALSE, VI_TRUE, VI_FALSE, VI_FALSE));
				}
				else if(detector == 5)  // CAverage
				{
					CHECKERR (rsspecan_ConfigureReceiverDetector (instrSession, 1, 1, VI_FALSE, VI_FALSE, VI_FALSE, VI_FALSE, VI_FALSE, VI_TRUE, VI_FALSE));
				}
				else if(detector == 6)  // CRMS
				{
					CHECKERR (rsspecan_ConfigureReceiverDetector (instrSession, 1, 1, VI_FALSE, VI_FALSE, VI_FALSE, VI_FALSE, VI_FALSE, VI_FALSE, VI_TRUE));
				}
				
				
				/*  */
                CHECKERR (rsspecan_RunReceiverMeasurement(instrSession));             
        
                enableTrace = VI_TRUE;
                
                /* enable mouse events*/
                SetWaitCursor (0);
                
				/* Read out the Scan Measurement */
                ReadScan ();
                  
            }
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Stop Button Callback
/* Purpose:  Stops the continuous measurement, in other cases has no sense
/*===========================================================================*/
int CVICALLBACK stopButton (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
         
    {
        case EVENT_COMMIT:
            
            /* stop measurements */
            enableTrace = VI_FALSE;
            
            /* cleanup specan driver session */
            cleanup(VI_TRUE);
                
            /* enable buttons for configuration */
            EnableButtons(VI_TRUE);
			
            break;
    }
    return 0;
}


int ReadScan (void)
{
    ViStatus temp_status  = VI_SUCCESS;
    ViInt32  actual_count = 0;
    ViInt32	 status_reg;
    ViChar   error_buffer [1024];
    ViInt32 i;
	ViInt32	 firstblock = 1;
    
    ViReal64 trace[6][ARRAY_SIZE];
    ViInt32  traceStatus=0,noofPoints=0;
    ViChar   statusData[ARRAY_SIZE];
	ViReal64 correction[2];
    
	
	//Clear graph
    temp_status = DeleteGraphPlot (panelHandle, PANEL_GRAPH, -1, VAL_DELAYED_DRAW);
    if( temp_status != 0 )
    {
        /* prompt CVI error message */
        MessagePopup ("Error", GetUILErrorString(temp_status));
    
        /* cleanup specan driver session */
        cleanup(VI_FALSE);
    
        /* do not free resource */
        enableTrace = VI_FALSE;
        return -1;
    }
        
    while(!(traceStatus & 0x1000))
    {
    	//Break if [Stop] has been hit
    	if(enableTrace==VI_FALSE) return -1;
    
    	//Read status register
    	rsspecan_getStatusRegister (instrSession, 1, &status_reg);

    	if(status_reg & 512)
    	{
	    	//Read Trace
	        temp_status = rsspecan_ReadScanMeasurement (instrSession, ARRAY_SIZE, &traceStatus, &noofPoints, statusData, 
	              trace[0], trace[1], trace[2], trace[3], trace[4], trace[5]);
	
			if( temp_status != VI_SUCCESS )
	        {
	            /* check error (specan driver) and prompt it */
	            checkError(temp_status);
            
	            /* do not free resource */
	            enableTrace = VI_FALSE;
	            return -1;
	        }
    	
	        //Plot the graph incrementally
	        PlotWaveform (panelHandle, PANEL_GRAPH, trace[0], noofPoints, VAL_DOUBLE,
	                        1.0, 0.0, freqstart + actual_count*freqstep, freqstep, VAL_THIN_LINE, VAL_SIMPLE_DOT,
	                        VAL_SOLID, 1, VAL_YELLOW);
        
			// The following block is of puer cosmetic nature. As between two plots there is one line missing,
			// we will plot this line afterwards. This only has to happen if its not the first block.
			if(firstblock) // If this is the first block read:
			{
				correction[0] = trace[0][noofPoints-1];  //Save the last value
				firstblock = 0;							 //First block finished
			}
			else		   // If its not the first block anymore
			{
				correction[1] = trace[0][0]; // Get the second point
		        PlotWaveform (panelHandle, PANEL_GRAPH, correction, 2, VAL_DOUBLE,
	                        1.0, 0.0, freqstart + (actual_count-1)*freqstep, freqstep, VAL_THIN_LINE, VAL_SIMPLE_DOT,
	                        VAL_SOLID, 1, VAL_YELLOW);
				correction[0] = trace[0][noofPoints-1]; // And save the last point for the next loop
			}
			
	        actual_count += noofPoints; // Number of points read altogether
	    }
	    else
	    {
	    	Rs_Delay(0.01);
	    }
    	
		ProcessSystemEvents();
   }
	
	/* stop measurements */
    enableTrace = VI_FALSE;
    
    /* cleanup specan driver session */
    cleanup(VI_TRUE);
        
    /* enable buttons for configuration */
    EnableButtons(VI_TRUE);
			

    return 0;
}




/*===========================================================================*/
/* Function: checkError
/* Purpose:  If passed status indicates an failure, the error message will
/*           be displayed.
/*===========================================================================*/
ViStatus checkError (ViStatus status)
{
    ViChar  error_message [STRING_LENGTH];
    ViChar  error_buffer  [STRING_LENGTH*2];
    ViChar* p2buf;
    ViInt32 error;
    
    if (status < VI_SUCCESS)
    {
        /* Converts a status code returned by an instrument driver function into a user-readable string */
        rsspecan_error_message (instrSession, status, error_message);
        p2buf = error_buffer + sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", status, error_message);
        
        
        /* This function reads an error code and a message from the instrument's error queue */
        rsspecan_error_query (instrSession, &error, error_message);
        
        SetWaitCursor (0);
        if (error != VI_SUCCESS)
        {
            sprintf (p2buf, "Secondary Error: 0x%08X, %s\n", error, error_message);
        }
        MessagePopup ("Error", error_buffer);
        
        /* close session */
        cleanup(VI_TRUE);
        
        /* enable buttons for configuration */
        EnableButtons(VI_TRUE);
    }
    return status;
}


/*===========================================================================*/
/* Function: EnableButtons
/* Purpose:  This function dis-/enables GUI while measuring/configuration.
/*===========================================================================*/
void EnableButtons(ViBoolean enable){

    SetCtrlAttribute (panelHandle, PANEL_START_FRQ, ATTR_DIMMED,    !enable);
    SetCtrlAttribute (panelHandle, PANEL_STOP_FRQ,  ATTR_DIMMED,    !enable);
    SetCtrlAttribute (panelHandle, PANEL_STEP_FRQ,  ATTR_DIMMED,    !enable);
    SetCtrlAttribute (panelHandle, PANEL_START,     ATTR_DIMMED,    !enable);
    SetCtrlAttribute (panelHandle, PANEL_RESET,     ATTR_DIMMED,    !enable);
    SetCtrlAttribute (panelHandle, PANEL_IDQUERY,   ATTR_DIMMED,    !enable);
    SetCtrlAttribute (panelHandle, PANEL_INSTR_DESC,ATTR_DIMMED,    !enable);
    SetCtrlAttribute (panelHandle, PANEL_CHECKBOX,  ATTR_DIMMED,    !enable);
	SetCtrlAttribute (panelHandle, PANEL_DETECTOR,  ATTR_DIMMED,    !enable);
    SetCtrlAttribute (panelHandle, PANEL_STOP,      ATTR_DIMMED,    enable);
}


/*===========================================================================*/
/* Function: Cleanup
/* Purpose:  Closes the VISA session to the instrument and in case of error
/*           queries the error and shows the dialog box with the passed error
/*           message. In case Device Clear is requested, it is necessary to 
/*           set the argument deviceClear to VI_TRUE.
/*===========================================================================*/
void cleanup (ViBoolean deviceClear)
{

    /* Device Clear is a low-level command and should be used in case the
     * instrument is waiting for Operation Complete to cancel the wait.
     * It is useful for instance in case of incorrect external trigger
     * when the instrument does not respond to any other command because of
     * waiting for trigger.
     */
    if (deviceClear)
    {
        viClear (instrSession);
        viPrintf (instrSession, "*CLS\n");
    }
    
    /* close instrument session */
    rsspecan_close (instrSession);
    instrSession = VI_NULL;
    
    /* no measurements */
    enableTrace  = VI_FALSE;
    
    /* enable GUI */
    EnableButtons (1);
    /* enable mouse events*/
    SetWaitCursor (0);
    
}

