#include <formatio.h>
#include "rsspecan.h"
#include <cvirte.h>     
#include <userint.h>
#include <visa.h>
#include "rsspecan_spurious_emissions_example.h"


#define ARRAY_SIZE    5000
#define STRING_LENGTH 512
#define WINDOW        1

/* Useful macros */
#define CHECKERR(fCal) \
    if (status = checkError((fCal)), status < VI_SUCCESS) \
        return status; else


/* Global variables */
static   ViInt32    panelHandle  = 0;
volatile ViBoolean  enableTrace  = VI_FALSE; 
static   ViSession  instrSession = VI_NULL;
    

/* Utility functions declarations */
ViStatus checkError(ViStatus status);
void cleanup (ViBoolean deviceClear);
void EnableButtons(ViBoolean enable);
int readData(void);

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    if ((panelHandle = LoadPanel (0, "rsspecan_spurious_emissions_example.uir", PANEL)) < 0)
        return -1;
    
    DisplayPanel (panelHandle);
    RunUserInterface ();
    DiscardPanel (panelHandle);
    
    return 0;
}

int CVICALLBACK panelCallback (int panel, int event, void *callbackData,
        int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_CLOSE:
            /* cleanup specan driver session */
            cleanup(VI_FALSE);
            
            QuitUserInterface (0);
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Start Button Callback
/* Purpose:  Configures the instrument and initiates the measurement
/*===========================================================================*/
int CVICALLBACK startButton (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViChar      instr_desc [STRING_LENGTH] = "";
    ViInt32     id               = 0;
    ViInt32     reset            = VI_FALSE;
    ViInt32     display          = VI_FALSE;
    ViStatus    status           = VI_SUCCESS;
	ViInt32		peakNum, detector;
	ViReal64    centerFreq, span, margin, startFreq, stopFreq, resolutionBW, videoBW, refLevel;
    
    switch (event)
    {
        case EVENT_COMMIT:     /* start button pressed */
            {
            
                /* block mouse events */
                SetWaitCursor (1);
                
                /* read values from panel*/
                GetCtrlVal (panelHandle, PANEL_INSTR_DESC, instr_desc);
                GetCtrlVal (panelHandle, PANEL_IDQUERY, &id);
                GetCtrlVal (panelHandle, PANEL_RESET, &reset);
                GetCtrlVal (panelHandle, PANEL_DISPLAYON, &display);
				GetCtrlVal (panelHandle, PANEL_CENTERFREQ, &centerFreq);
				GetCtrlVal (panelHandle, PANEL_SPAN, &span);
				GetCtrlVal (panelHandle, PANEL_MARGIN, &margin);
                GetCtrlVal (panelHandle, PANEL_PEAKNUM, &peakNum);
				GetCtrlVal (panelHandle, PANEL_STARTFREQ, &startFreq);
				GetCtrlVal (panelHandle, PANEL_STOPFREQ, &stopFreq);
				GetCtrlVal (panelHandle, PANEL_RBW, &resolutionBW);
				GetCtrlVal (panelHandle, PANEL_VBW, &videoBW);
                GetCtrlVal (panelHandle, PANEL_DETECTOR, &detector);
                GetCtrlVal (panelHandle, PANEL_REFLEVEL, &refLevel);

                /* disable buttons for configuration */
                EnableButtons(VI_FALSE);
                
                /* initialize driver */
                CHECKERR (rsspecan_init (instr_desc, id, reset, &instrSession));
				
				/* switch device display on/off */
                CHECKERR( rsspecan_ConfigureDisplayUpdate(instrSession, display));
                
                /* Set instrument to single sweep mode - continous mode is done
                 * programmatically because this is the only way how to
                 * synchronize measurement using *OPC
                 */
                CHECKERR (rsspecan_ConfigureSignalStatisticSweep(instrSession, VI_FALSE));
                
				/* configure the center frequency and span */
				CHECKERR (rsspecan_ConfigureFrequencyCenterSpan (instrSession, WINDOW, centerFreq,
																 span));
				
				
				/* switch on the spurious emissions measurement */
				CHECKERR (rsspecan_ConfigureSEState (instrSession, VI_TRUE));
				
				
				/* configure the peak search for spurious emission measurement */ 
				CHECKERR (rsspecan_ConfigureSEPeakSearch (instrSession, WINDOW, margin, peakNum));
				
				/* configure the start and stop frequency for the specified range */ 
				CHECKERR (rsspecan_ConfigureSESweepListStartStop (instrSession, 1, startFreq,
																  stopFreq));
				
				/* configure the resolution and video bandwidth values for the specified range */ 
				CHECKERR (rsspecan_ConfigureSESweepListBandwidth (instrSession, 1, resolutionBW,
																  videoBW));
				
				/* configure the detector value for the specified range */ 
				CHECKERR (rsspecan_ConfigureSESweepListDetector (instrSession, 1, detector));
				
				/* configure the reference level for the specified range */ 
				CHECKERR (rsspecan_ConfigureSESweepListReferenceLevel (instrSession, 1,
																	   refLevel));
				
				
				/* perform measurement */
				readData();
				
				/* close session */
				cleanup(VI_FALSE);
            
            }
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Read Data
/* Purpose:  Reads data from the instrument
/*===========================================================================*/
int readData (void)
{
    ViStatus temp_status  = VI_SUCCESS;
	ViStatus status       = VI_SUCCESS;
	ViInt32  timeout;
	ViInt32  returnedValues, peakNum;
	ViReal64 frequency[ARRAY_SIZE], level[ARRAY_SIZE], deltaLimit[ARRAY_SIZE];
	int i;
    
	
	/* read value from panel */
	GetCtrlVal (panelHandle, PANEL_TIMEOUT, &timeout);
	GetCtrlVal (panelHandle, PANEL_PEAKNUM, &peakNum);
	
	/* start a new spurious measurement */
	CHECKERR (rsspecan_InitiateSpurious (instrSession, WINDOW, timeout));
	
	/* search the peaks in spurious emissions data */
	CHECKERR (rsspecan_SESearchPeak (instrSession));
	
	/* read the spurious emissions peak list */
	CHECKERR (rsspecan_ReadSEPeakList (instrSession, peakNum, frequency, level,
									   deltaLimit, &returnedValues));
	
	/* create right number of rows in table */
	DeleteTableRows (panelHandle, PANEL_TABLE, 1, -1);
	InsertTableRows (panelHandle, PANEL_TABLE, -1, returnedValues, -1);
	
	/* show result in front panel table */ 
	for (i=0; i<returnedValues; i++)
	{
		
		SetTableCellVal(panelHandle, PANEL_TABLE, MakePoint (1,i+1),frequency[i]);
		SetTableCellVal(panelHandle, PANEL_TABLE, MakePoint (2,i+1),level[i]);  
		SetTableCellVal(panelHandle, PANEL_TABLE, MakePoint (3,i+1),deltaLimit[i]); 
	}
	
    return 0;
}


/*===========================================================================*/
/* Function: checkError
/* Purpose:  If passed status indicates an failure, the error message will
/*           be displayed.
/*===========================================================================*/
ViStatus checkError (ViStatus status)
{
    ViChar  error_message [STRING_LENGTH];
    ViChar  error_buffer  [STRING_LENGTH*2];
    ViChar* p2buf;
    ViInt32 error;
    
    if (status < VI_SUCCESS)
    {
        /* Converts a status code returned by an instrument driver function into a user-readable string */
        rsspecan_error_message (instrSession, status, error_message);
        p2buf = error_buffer + sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", status, error_message);
        
        
        /* This function reads an error code and a message from the instrument's error queue */
        rsspecan_error_query (instrSession, &error, error_message);
        
        SetWaitCursor (0);
        if (error != VI_SUCCESS)
        {
            sprintf (p2buf, "Secondary Error: 0x%08X, %s\n", error, error_message);
        }
        MessagePopup ("Error", error_buffer);
        
        /* close session */
        cleanup(VI_TRUE);
        
        /* enable buttons for configuration */
        EnableButtons(VI_TRUE);
    }
    return status;
}


/*===========================================================================*/
/* Function: EnableButtons
/* Purpose:  This function dis-/enables GUI while measuring/configuration.
/*===========================================================================*/
void EnableButtons(ViBoolean enable){

    SetCtrlAttribute (panelHandle, PANEL_START,     ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_RESET,     ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_IDQUERY,   ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_INSTR_DESC,ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_DISPLAYON, ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_CENTERFREQ,ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_SPAN, 		ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_TIMEOUT, 	ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_MARGIN, 	ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_PEAKNUM, 	ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_STARTFREQ, ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_STOPFREQ, 	ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_RBW, 		ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_VBW, 		ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_DETECTOR, 	ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_REFLEVEL, 	ATTR_DIMMED,!enable);
    
    
}


/*===========================================================================*/
/* Function: Cleanup
/* Purpose:  Closes the VISA session to the instrument and in case of error
/*           queries the error and shows the dialog box with the passed error
/*           message. In case Device Clear is requested, it is necessary to 
/*           set the argument deviceClear to VI_TRUE.
/*===========================================================================*/
void cleanup (ViBoolean deviceClear)
{

    /* Device Clear is a low-level command and should be used in case the
     * instrument is waiting for Operation Complete to cancel the wait.
     * It is useful for instance in case of incorrect external trigger
     * when the instrument does not respond to any other command because of
     * waiting for trigger.
     */
    if (deviceClear)
    {
        viClear (instrSession);
        viPrintf (instrSession, "*CLS\n");
    }
    
    /* close instrument session */
    rsspecan_close (instrSession);
    instrSession = VI_NULL;
    
    /* no measurements */
    enableTrace  = VI_FALSE;
    
    /* enable GUI */
    EnableButtons (1);
    /* enable mouse events*/
    SetWaitCursor (0);
    
}


