#include "rsspecan.h"
#include <cvirte.h>     
#include <userint.h>
#include <visa.h>
#include "rsspecan_sync_with_opc_example.h"


#define ARRAY_SIZE    5000
#define STRING_LENGTH 512

/* Useful macros */
#define CHECKERR(fCal) \
    if (status = checkError((fCal)), status < VI_SUCCESS) \
        return status; else


/* Global variables */
static   ViInt32    panelHandle  = 0;
static   ViSession  instrSession = VI_NULL;
ViReal64            amplitude [ARRAY_SIZE];
    

/* Utility functions declarations */
ViStatus checkError(ViStatus status);
void cleanup (ViBoolean deviceClear);
void EnableButtons(ViBoolean enable);
int readData(void); 

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    if ((panelHandle = LoadPanel (0, "rsspecan_sync_with_opc_example.uir", PANEL)) < 0)
        return -1;
    
    DisplayPanel (panelHandle);
    RunUserInterface ();
    DiscardPanel (panelHandle);
    
    return 0;
}

int CVICALLBACK panelCallback (int panel, int event, void *callbackData,
        int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_CLOSE:
            /* cleanup specan driver session */
            cleanup(VI_FALSE);
            
            QuitUserInterface (0);
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Start Button Callback
/* Purpose:  Configures the instrument and initiates the measurement
/*===========================================================================*/
int CVICALLBACK startButton (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViChar      instr_desc [STRING_LENGTH] = "";
    ViInt32     id               = 0;
    ViInt32     reset            = VI_FALSE;
    ViInt32     display          = VI_FALSE;
    ViStatus    status           = VI_SUCCESS;
    
    switch (event)
    {
        case EVENT_COMMIT:     /* start button pressed */
            {
            
                /* block mouse events */
                SetWaitCursor (1);
                
                /* read values from panel*/
                GetCtrlVal (panelHandle, PANEL_INSTR_DESC, instr_desc);
                GetCtrlVal (panelHandle, PANEL_IDQUERY, &id);
                GetCtrlVal (panelHandle, PANEL_RESET, &reset);

                /* disable buttons for configuration */
                EnableButtons(VI_FALSE);
                
                /* initialize driver */
                CHECKERR (rsspecan_init (instr_desc, id, reset, &instrSession));
				
                /* Set instrument to single sweep mode - continous mode is done
                 * programmatically because this is the only way how to
                 * synchronize measurement using *OPC
                 */
                CHECKERR (rsspecan_ConfigureSignalStatisticSweep(instrSession, VI_FALSE));
				
				/* initiate measurement and show data */
				readData();
				
				/* cleanup specan driver session */
            	cleanup(VI_FALSE);
            }
            break;
    }
    return 0;
}



/*===========================================================================*/
/* Function: Read Data
/* Purpose:  Read data from the instrument and plot data onto graph
/*===========================================================================*/
int readData()
{
    ViStatus temp_status  = VI_SUCCESS;
    ViInt32  timeout      = 5000;               /* default value */
    ViInt32  actual_count = 0;
    ViChar   error_buffer [1024];
    int i;
	
	
			/* Initiates a signal acquisition based on the present instrument configuration. 
             * It then waits for the acquisition to complete, and returns the trace as an 
             * array of amplitude values. */
			temp_status = rsspecan_ReadYTrace (instrSession, 1, 1, timeout,
											   ARRAY_SIZE, &actual_count,
											   amplitude);
            if( temp_status != VI_SUCCESS )
            {
                /* check error (specan driver) and prompt it */
                checkError(temp_status);
				
                return -1;
            }
            
            /* deletes all points of the graph plot */
			temp_status = DeleteGraphPlot (panelHandle, PANEL_GRAPH, -1,
										   VAL_DELAYED_DRAW);
            if( temp_status != 0 )
            {
                /* prompt CVI error message */
                MessagePopup ("Error", GetUILErrorString(temp_status));
                
                /* cleanup specan driver session */
                cleanup(VI_FALSE);
                
                return -1;
            }
            
            /* Plots a waveform onto a graph control */
			temp_status = PlotWaveform (panelHandle, PANEL_GRAPH, amplitude,
										actual_count, VAL_DOUBLE, 1.0, 0.0, 0.0,
										1, VAL_THIN_LINE, VAL_SIMPLE_DOT,
										VAL_SOLID, 1, VAL_YELLOW);
            if( temp_status < 0 )
            {
                /* prompt CVI error message */
                MessagePopup ("Error", GetUILErrorString(temp_status));
                
                /* cleanup specan driver session */
                cleanup(VI_FALSE);
                
                return -1;
            }
			
			return 0;

}

/*===========================================================================*/
/* Function: checkError
/* Purpose:  If passed status indicates an failure, the error message will
/*           be displayed.
/*===========================================================================*/
ViStatus checkError (ViStatus status)
{
    ViChar  error_message [STRING_LENGTH];
    ViChar  error_buffer  [STRING_LENGTH*2];
    ViChar* p2buf;
    ViInt32 error;
    
    if (status < VI_SUCCESS)
    {
        /* Converts a status code returned by an instrument driver function into a user-readable string */
        rsspecan_error_message (instrSession, status, error_message);
        p2buf = error_buffer + sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", status, error_message);
        
        
        /* This function reads an error code and a message from the instrument's error queue */
        rsspecan_error_query (instrSession, &error, error_message);
        
        SetWaitCursor (0);
        if (error != VI_SUCCESS)
        {
            sprintf (p2buf, "Secondary Error: 0x%08X, %s\n", error, error_message);
        }
        MessagePopup ("Error", error_buffer);
        
        /* close session */
        cleanup(VI_TRUE);
        
        /* enable buttons for configuration */
        EnableButtons(VI_TRUE);
    }
    return status;
}


/*===========================================================================*/
/* Function: EnableButtons
/* Purpose:  This function dis-/enables GUI while measuring/configuration.
/*===========================================================================*/
void EnableButtons(ViBoolean enable){

    SetCtrlAttribute (panelHandle, PANEL_START, ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_RESET, ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_IDQUERY,   ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_INSTR_DESC,ATTR_DIMMED,!enable);
    //SetCtrlAttribute (panelHandle, PANEL_STOP,      ATTR_DIMMED, enable);
    
    
}


/*===========================================================================*/
/* Function: Cleanup
/* Purpose:  Closes the VISA session to the instrument and in case of error
/*           queries the error and shows the dialog box with the passed error
/*           message. In case Device Clear is requested, it is necessary to 
/*           set the argument deviceClear to VI_TRUE.
/*===========================================================================*/
void cleanup (ViBoolean deviceClear)
{

    /* Device Clear is a low-level command and should be used in case the
     * instrument is waiting for Operation Complete to cancel the wait.
     * It is useful for instance in case of incorrect external trigger
     * when the instrument does not respond to any other command because of
     * waiting for trigger.
     */
    if (deviceClear)
    {
        viClear (instrSession);
        viPrintf (instrSession, "*CLS\n");
    }
    
    /* close instrument session */
    rsspecan_close (instrSession);
    instrSession = VI_NULL;
    
    /* enable GUI */
    EnableButtons (1);
    /* enable mouse events*/
    SetWaitCursor (0);
    
}
