#include "rsspecan.h"
#include <cvirte.h>     
#include <userint.h>
#include <visa.h>
#include "rsspecan_trace_read_example.h"


#define ARRAY_SIZE    5000         /* for rsspecan_ReadYTrace() varable: amplitude */
#define STRING_LENGTH 512
#define WINDOW        1

/* Useful macros */
#define CHECKERR(fCal) \
    if (status = checkError((fCal)), status < VI_SUCCESS) \
        return status; else


/* Global variables */
static   ViInt32    panelHandle  = 0;
static   ViSession  instrSession = VI_NULL;
ViReal64            amplitude [ARRAY_SIZE];
    

/* Utility functions declarations */
ViStatus checkError(ViStatus status);
void cleanup (ViBoolean deviceClear);
void EnableButtons(ViBoolean enable);

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    if ((panelHandle = LoadPanel (0, "rsspecan_trace_read_example.uir", PANEL)) < 0)
        return -1;
    
    DisplayPanel (panelHandle);
    RunUserInterface ();
    DiscardPanel (panelHandle);
    
    return 0;
}

int CVICALLBACK panelCallback (int panel, int event, void *callbackData,
        int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_CLOSE:
            /* cleanup specan driver session */
            cleanup(VI_FALSE);
            
            QuitUserInterface (0);
            break;
    }
    return 0;
}


/*===========================================================================*/
/* Function: Start Button Callback
/* Purpose:  Configures the instrument according to selected measurement
/*           parameters and enables the measurement loop
/*===========================================================================*/
int CVICALLBACK startButton (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViChar      instr_desc [STRING_LENGTH] = "";
    ViChar      instr_modl [STRING_LENGTH] = "";
    ViReal64    center           = 0.0;
    ViReal64    span             = 0.0;
    ViInt32     id               = 0;
    ViInt32     reset            = VI_FALSE;
    ViStatus    status           = VI_SUCCESS;
    ViInt32     timeout      = 5000;               /* default value */
    ViInt32     actual_count = 0;
    ViChar      error_buffer [1024];
    
    switch (event)
    {
        case EVENT_COMMIT:     /* start button pressed */
            {
            
                /* block mouse events */
                SetWaitCursor (1);
                
                /* read values from panel*/
                GetCtrlVal (panelHandle, PANEL_INSTR_DESC, instr_desc);
                GetCtrlVal (panelHandle, PANEL_IDQUERY, &id);
                GetCtrlVal (panelHandle, PANEL_RESET, &reset);
                GetCtrlVal (panelHandle, PANEL_CENTER, &center);
                GetCtrlVal (panelHandle, PANEL_SPAN, &span);
            
                /* disable buttons for configuration */
                EnableButtons(VI_FALSE);
                
                /* initialize driver */
                CHECKERR (rsspecan_init (instr_desc, id, reset, &instrSession));
                
                /* query instrument model */
                CHECKERR (rsspecan_GetAttributeViString (instrSession, "", RS_ATTR_INSTRUMENT_MODEL, 
                    STRING_LENGTH, instr_modl));
                
                /*
                 * do not switch to spectrum analyzer mode if an FMU baseband analyzer is
                 * connected. For this example the standard FFT-mode on the FMU is used
                 */
                if(0 != strcmp("FMU", instr_modl))
                    CHECKERR (rsspecan_SetSANMode (instrSession));   

                    
                /* configure specan device: center frequency and span */
                CHECKERR (rsspecan_ConfigureFrequencyCenterSpan (instrSession, WINDOW, center, span));
                
                /* Set instrument to single sweep mode - continous mode is done
                 * programmatically because this is the only way how to
                 * synchronize measurement using *OPC
                 */
                CHECKERR (rsspecan_ConfigureSignalStatisticSweep(instrSession, VI_FALSE));
                
                /* Initiates a signal acquisition based on the present instrument configuration. 
                 * It then waits for the acquisition to complete, and returns the trace as an 
                 * array of amplitude values. */
                status = rsspecan_ReadYTrace (instrSession, 1, 1, timeout, ARRAY_SIZE, &actual_count, amplitude);
                if( status != VI_SUCCESS )
                {
                    /* check error (specan driver) and prompt it */
                    checkError(status);
            
                    return -1;
                }
        
                /* deletes all points of the graph plot */
                status = DeleteGraphPlot (panelHandle, PANEL_GRAPH, -1, VAL_DELAYED_DRAW);
                if( status != 0 )
                {
                    /* prompt CVI error message */
                    MessagePopup ("Error", GetUILErrorString(status));
            
                    /* cleanup specan driver session */
                    cleanup(VI_FALSE);
            
                    return -1;
                }
        
                /* Plots a waveform onto a graph control */
                status = PlotWaveform (panelHandle, PANEL_GRAPH, amplitude, actual_count, VAL_DOUBLE,
                                1.0, 0.0, 0.0, 1, VAL_THIN_LINE, VAL_SIMPLE_DOT,
                                VAL_SOLID, 1, VAL_YELLOW);
                if( status < 0 )
                {
                    /* prompt CVI error message */
                    MessagePopup ("Error", GetUILErrorString(status));
            
                    /* cleanup specan driver session */
                    cleanup(VI_FALSE);
            
                    return -1;
                }
        
                /* enable mouse events*/
                SetWaitCursor (0);
                
                /* disable buttons for configuration */
                EnableButtons(VI_TRUE);
            }
            break;
            
    }
    return 0;
}



/*===========================================================================*/
/* Function: checkError
/* Purpose:  If passed status indicates an failure, the error message will
/*           be displayed.
/*===========================================================================*/
ViStatus checkError (ViStatus status)
{
    ViChar  error_message [STRING_LENGTH];
    ViChar  error_buffer  [STRING_LENGTH*2];
    ViChar* p2buf;
    ViInt32 error;
    
    if (status < VI_SUCCESS)
    {
        /* Converts a status code returned by an instrument driver function into a user-readable string */
        rsspecan_error_message (instrSession, status, error_message);
        p2buf = error_buffer + sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", status, error_message);
        
        
        /* This function reads an error code and a message from the instrument's error queue */
        rsspecan_error_query (instrSession, &error, error_message);
        
        SetWaitCursor (0);
        if (error != VI_SUCCESS)
        {
            sprintf (p2buf, "Secondary Error: 0x%08X, %s\n", error, error_message);
        }
        MessagePopup ("Error", error_buffer);
        
        /* close session */
        cleanup(VI_TRUE);
        
        /* enable buttons for configuration */
        EnableButtons(VI_TRUE);
    }
    return status;
}


/*===========================================================================*/
/* Function: EnableButtons
/* Purpose:  This function dis-/enables GUI while measuring/configuration.
/*===========================================================================*/
void EnableButtons(ViBoolean enable){

    SetCtrlAttribute (panelHandle, PANEL_CENTER,ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_SPAN,  ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_START, ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_RESET, ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_IDQUERY,   ATTR_DIMMED,!enable);
    SetCtrlAttribute (panelHandle, PANEL_INSTR_DESC,ATTR_DIMMED,!enable);
}


/*===========================================================================*/
/* Function: Cleanup
/* Purpose:  Closes the VISA session to the instrument and in case of error
/*           queries the error and shows the dialog box with the passed error
/*           message. In case Device Clear is requested, it is necessary to 
/*           set the argument deviceClear to VI_TRUE.
/*===========================================================================*/
void cleanup (ViBoolean deviceClear)
{

    /* Device Clear is a low-level command and should be used in case the
     * instrument is waiting for Operation Complete to cancel the wait.
     * It is useful for instance in case of incorrect external trigger
     * when the instrument does not respond to any other command because of
     * waiting for trigger.
     */
    if (deviceClear)
    {
        viClear (instrSession);
        viPrintf (instrSession, "*CLS\n");
    }
    
    /* close instrument session */
    rsspecan_close (instrSession);
    instrSession = VI_NULL;
    
    /* enable GUI */
    EnableButtons (1);
    /* enable mouse events*/
    SetWaitCursor (0);
    
}
